package cooperari.pppj14;

import static cooperari.junit.CTestRunner.runThreads;
import static org.junit.Assert.assertEquals;

import org.junit.BeforeClass;
import org.junit.Test;

/**
 * Base class with the test logic.
 * Do not execute it directly.
 * 
 * @author Eduardo Marques, DI/FCUL, 2014.
 */
public class BaseTest {

  /** 
   * Number of threads 
   */
  private static int N = 4;

  @BeforeClass
  public static void setupAllTests() {
    String p = System.getProperty("N");
    if (p != null) {
      N = Integer.parseInt(p);
    }
  }
  
  /**
   * Client thread.
   */
  private static class Client implements Runnable {
    private Semaphore sem;
    public Client(Semaphore s) {
      sem = s;
    }
    public void run() {
      try {
        sem.down();
        sem.up();
      } catch (InterruptedException e) { /* ignored */
      }
    }
  }
  /**
   * Base test method
   * @param s A {@code Semaphore object}. 
   */
  public void test(Semaphore s) {
    Runnable[] c = new Runnable[N];
    for (int i = 0; i < N; i++) {
      c[i] = new Client(s);
    }
    runThreads(c);
    assertEquals(N - 1, s.getValue());
  }

  @Test
  public final void testBug1() {
    test( new BuggySemaphore1(N - 1) ); 
  }
  
  @Test
  public final void testBug2() {
    test( new BuggySemaphore2(N - 1) ); 
  }
  @Test
  public final void testBug3() {
    test( new BuggySemaphore3(N - 1) ); 
  }
  @Test
  public final void testBug4() {
    test( new BuggySemaphore4(N - 1) ); 
  }
  @Test
  public final void testBug5() {
    test( new BuggySemaphore5(N - 1) ); 
  }
  @Test
  public final void testCorrectSemaphore() {
    test( new CorrectSemaphore(N - 1) ); 
  }
  
}
