package cooperari.aspectj;

import cooperari.coverage.Noise;

/**
 * Instrumentation for virtualization of java.lang.Object
 * and monitor locking and unlocking.
 */
aspect VJavaLangObject {

  before () : lock() {
    Noise.tick();
  }

  before () : unlock() {
    Noise.tick();
  }

  void around(Object o) : call(void Object.wait()) && target(o) {
    Noise.tick();
    proceed(o);
  }

  void around(Object o, long timeout) 
  : call(void Object.wait(long)) && target(o) && args(timeout) {
    Noise.tick();
    proceed(o,timeout);
  }

  void around(Object o, long timeout, int nano) 
  : call(void Object.wait(long, int)) && target(o) && args(timeout,nano) {
    Noise.tick();
    proceed(o,timeout,nano);
  }

  void around(Object o) : call(void Object.notify()) && target(o) {
    Noise.tick();
    proceed(o);
  }
  
  void around(Object o) : call(void Object.notifyAll()) && target(o) {
    Noise.tick();
    proceed(o);
  }
}
