package cooperari.aspectj;

import cooperari.coverage.Noise;


/**
 * Instrumentation for virtualization of java.lang.Thread
 * operations.
 */
aspect VJavaLangThread {
  
  Thread.State around(Thread otherThread) : call(Thread.State Thread.getState()) && target(otherThread) {
    return proceed(otherThread);
  }
  
  boolean around(Object o) : call(boolean Thread.holdsLock(Object)) && args(o) {
    return proceed(o);
  }

  void around(long millis) : call(void Thread.sleep(long)) && args(millis) {
    Noise.tick();
    proceed(millis);
  }

  void around(long millis, int nano) : call(void Thread.sleep(long,int)) && args(millis,nano) {
    Noise.tick();
    proceed(millis, nano);
  }

  void around() : call(void Thread.yield())  {
    Noise.tick();
    proceed();
  }
  
  void around(Thread otherThread) : call(void Thread.interrupt()) && target(otherThread) {
    Noise.tick();
    proceed(otherThread);
  }

  boolean around() : call(boolean Thread.interrupted())  {
    Noise.tick();
    return proceed();
  }
  
  boolean around(Thread otherThread) : call(boolean Thread.isInterrupted()) && target(otherThread) {
    Noise.tick();
    return proceed(otherThread);
  }
  
  void around(Thread otherThread) : call(void Thread.join()) && target(otherThread) {
    Noise.tick();
    proceed(otherThread);
  }

  void around(Thread otherThread, long millis) 
  : call(void Thread.join(long)) && target(otherThread) && args(millis) {
    Noise.tick();
    proceed(otherThread, millis);
  }

  void around(Thread otherThread, long millis, int nano) 
  : call(void Thread.join(long, int)) && target(otherThread) && args(millis,nano) {
    Noise.tick();
    proceed(otherThread, millis,nano);
  }
  
  void around(Thread otherThread) : call(void Thread.stop()) && target(otherThread) {
    Noise.tick();
    proceed(otherThread);
  }
}
