package cooperari.aspectj;

import org.aspectj.lang.reflect.SourceLocation;

import cooperari.core.CThread;

/**
 * Instrumentation for virtualization of java.lang.Object
 * and monitor locking and unlocking.
 */
aspect VJavaLangObject {

  void around (Object o) : lock() && args(o) {
    CThread t = CThread.intercept(thisJoinPoint, o.getClass());
    if (t != null) {
      t.cMonitorEnter(o);
    } else {
      proceed(o);
    }
  }

  void around (Object o) : unlock() && args(o) {
    CThread t = CThread.intercept(thisJoinPoint, o.getClass());
    if (t != null) {
      t.cMonitorExit(o);
    } else {
      proceed(o);
    }
  }

  void around(Object o) : call(void Object.wait()) && target(o) {
    CThread t = CThread.intercept(thisJoinPoint);
    if (t != null) {
      t.cWait(o, 0);
    } else {
      proceed(o);
    }
  }

  void around(Object o, long timeout) 
  : call(void Object.wait(long)) && target(o) && args(timeout) {
    CThread t = CThread.intercept(thisJoinPoint);
    if (t != null) {
      t.cWait(o, timeout * 1000000L);
    } else {
      proceed(o, timeout);
    }
  }

  void around(Object o, long timeout, int nano) 
  : call(void Object.wait(long, int)) && target(o) && args(timeout,nano) {
    CThread t = CThread.intercept(thisJoinPoint);
    if (t != null) {
      t.cWait(o, timeout * 1000000L + nano);
    } else {
      proceed(o, timeout, nano);
    }
  }

  void around(Object o) : call(void Object.notify()) && target(o) {
    CThread t = CThread.intercept(thisJoinPoint);
    if (t != null) {
      t.cNotify(o);
    } else {
      proceed(o);
    }
  }
  void around(Object o) : call(void Object.notifyAll()) && target(o) {
    CThread t = CThread.intercept(thisJoinPoint);
    if (t != null) {
      t.cNotifyAll(o);
    } else {
      proceed(o);
    }
  }
}
