package cooperari.aspectj;

import cooperari.core.CThread;


/**
 * Instrumentation for virtualization of java.lang.Thread
 * operations.
 */
aspect VJavaLangThread {
  
  Thread.State around(Thread otherThread) : call(Thread.State Thread.getState()) && target(otherThread) {
    if (otherThread instanceof CThread) {
      return ((CThread)otherThread).getVirtualState();
    } else {
      return proceed(otherThread);
    }
  }
  
  boolean around(Object o) : call(boolean Thread.holdsLock(Object)) && args(o) {
    CThread thisThread = CThread.intercept(thisJoinPoint);
    if (thisThread != null) {
      return thisThread.cHoldsLock(o);
    } else {
      return proceed(o);
    }
  }

  void around(long millis) : call(void Thread.sleep(long)) && args(millis) {
    CThread thisThread = CThread.intercept(thisJoinPoint);
    if (thisThread != null) {
      thisThread.cSleep(millis * 1000000L);
    } else {
      proceed(millis);
    }
  }

  void around(long millis, int nano) : call(void Thread.sleep(long,int)) && args(millis,nano) {
    CThread thisThread = CThread.intercept(thisJoinPoint);
    if (thisThread != null) {
      thisThread.cSleep(millis * 1000000L + nano);
    } else {
      proceed(millis,nano);
    }
  }

  void around() : call(void Thread.yield())  {
    CThread thisThread = CThread.intercept(thisJoinPoint);
    if (thisThread != null) {
      thisThread.cYield();
    } else {
      proceed();
    }
  }
  
  void around(Thread otherThread) : call(void Thread.interrupt()) && target(otherThread) {
    CThread thisThread = CThread.intercept(thisJoinPoint);
    if (thisThread != null) {
      thisThread.cInterrupt((CThread) otherThread);
    } else {
      proceed(otherThread);
    }
  }

  boolean around() : call(boolean Thread.interrupted())  {
    // Note that Thread.interrupted() is a static method 
    // that checks and clears the interrupted status of the current thread.
    CThread thisThread = CThread.intercept(thisJoinPoint);
    if (thisThread != null) {
      return thisThread.cTestAndClearInterruptStatus();
    } else {
      return proceed();
    }
  }
  
  boolean around(Thread otherThread) : call(boolean Thread.isInterrupted()) && target(otherThread) {
    // This is not supposed to clear the interrupt status of targethisThread.
    CThread thisThread = CThread.intercept(thisJoinPoint);
    if (thisThread != null) {
      return thisThread.cTestIfInterrupted((CThread)otherThread);
    } else {
      return proceed(otherThread);
    }
  }
  
  void around(Thread otherThread) : call(void Thread.join()) && target(otherThread) {
    CThread thisThread = CThread.intercept(thisJoinPoint);
    if (thisThread != null) {
      thisThread.cJoin((CThread) otherThread, 0L);
    } else {
      proceed(otherThread);
    }
  }

  void around(Thread otherThread, long millis) 
  : call(void Thread.join(long)) && target(otherThread) && args(millis) {
    CThread thisThread = CThread.intercept(thisJoinPoint);
    if (thisThread != null) {
      thisThread.cJoin((CThread) otherThread, millis * 1000000L);
    } else {
      proceed(otherThread, millis);
    }
  }

  void around(Thread otherThread, long millis, int nano) 
  : call(void Thread.join(long, int)) && target(otherThread) && args(millis,nano) {
    CThread thisThread = CThread.intercept(thisJoinPoint);
    if (thisThread != null) {
      thisThread.cJoin((CThread) otherThread, millis * 1000000L + nano);
    } else {
      proceed(otherThread, millis, nano);
    }
  }
  
  void around(Thread otherThread) : call(void Thread.stop()) && target(otherThread) {
    CThread thisThread = CThread.intercept(thisJoinPoint);
    if (thisThread != null) {
      thisThread.cStop((CThread) otherThread);
    } else {
      proceed(otherThread);
    }
  }
}
